@@ -1,14 +1,14 @@
 #[cfg(not(feature = "library"))]
 use cosmwasm_std::entry_point;
 use cosmwasm_std::{
-    from_slice, to_binary, to_vec, Binary, ContractInfoResponse, ContractResult, Deps, DepsMut,
-    Empty, Env, MessageInfo, QueryRequest, StdResult, SystemResult, WasmMsg, WasmQuery,
+    from_slice, to_binary, to_vec, Binary, ContractInfoResponse, ContractResult, CustomQuery, Deps,
+    DepsMut, Empty, Env, MessageInfo, QueryRequest, StdResult, SystemResult, WasmMsg, WasmQuery,
 };
 
 use cw2::set_contract_version;
 use tg_bindings::{
     request_privileges, BlockParams, ConsensusParams, EvidenceParams, GovProposal, Privilege,
-    PrivilegeChangeMsg, TgradeMsg, TgradeSudoMsg,
+    PrivilegeChangeMsg, TgradeMsg, TgradeQuery, TgradeSudoMsg,
 };
 use tg_utils::ensure_from_older_version;
 
@@ -16,9 +16,10 @@ use crate::msg::{ExecuteMsg, InstantiateMsg, QueryMsg, ValidatorProposal};
 use crate::ContractError;
 
 use tg_voting_contract::{
-    close as execute_close, list_proposals, list_voters, list_votes, mark_executed,
-    propose as execute_propose, query_group_contract, query_proposal, query_rules, query_vote,
-    query_voter, reverse_proposals, vote as execute_vote,
+    close as execute_close, execute_text, list_proposals, list_text_proposals, list_voters,
+    list_votes, list_votes_by_voter, mark_executed, propose as execute_propose,
+    query_group_contract, query_proposal, query_rules, query_vote, query_voter, reverse_proposals,
+    vote as execute_vote,
 };
 
 pub type Response = cosmwasm_std::Response<TgradeMsg>;
@@ -30,7 +31,7 @@ const CONTRACT_VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[cfg_attr(not(feature = "library"), entry_point)]
 pub fn instantiate(
-    deps: DepsMut,
+    deps: DepsMut<TgradeQuery>,
     _env: Env,
     _info: MessageInfo,
     msg: InstantiateMsg,
@@ -41,7 +42,7 @@ pub fn instantiate(
 
 #[cfg_attr(not(feature = "library"), entry_point)]
 pub fn execute(
-    deps: DepsMut,
+    deps: DepsMut<TgradeQuery>,
     env: Env,
     info: MessageInfo,
     msg: ExecuteMsg,
@@ -56,24 +57,34 @@ pub fn execute(
         } => {
             // Migrate contract needs confirming that sender (proposing member) is an admin
             // of target contract
-            if let ValidatorProposal::MigrateContract { ref contract, .. } = proposal {
+            if let ValidatorProposal::MigrateContract {
+                ref contract,
+                ref migrate_msg,
+                ..
+            } = proposal
+            {
                 confirm_admin_in_contract(deps.as_ref(), &env, contract.clone())?;
+                if migrate_msg.is_empty() {
+                    return Err(ContractError::MigrateMsgCannotBeEmptyString {});
+                }
             };
             execute_propose(deps, env, info, title, description, proposal)
                 .map_err(ContractError::from)
         }
         Vote { proposal_id, vote } => {
-            execute_vote::<ValidatorProposal>(deps, env, info, proposal_id, vote)
+            execute_vote::<ValidatorProposal, TgradeQuery>(deps, env, info, proposal_id, vote)
                 .map_err(ContractError::from)
         }
         Execute { proposal_id } => execute_execute(deps, env, info, proposal_id),
-        Close { proposal_id } => execute_close::<ValidatorProposal>(deps, env, info, proposal_id)
-            .map_err(ContractError::from),
+        Close { proposal_id } => {
+            execute_close::<ValidatorProposal, TgradeQuery>(deps, env, info, proposal_id)
+                .map_err(ContractError::from)
+        }
     }
 }
 
-fn confirm_admin_in_contract(
-    deps: Deps,
+fn confirm_admin_in_contract<Q: CustomQuery>(
+    deps: Deps<Q>,
     env: &Env,
     contract_addr: String,
 ) -> Result<(), ContractError> {
@@ -105,39 +116,49 @@ fn confirm_admin_in_contract(
     ))
 }
 
-pub fn execute_execute(
-    deps: DepsMut,
+pub fn execute_execute<Q: CustomQuery>(
+    deps: DepsMut<Q>,
     env: Env,
     info: MessageInfo,
     proposal_id: u64,
 ) -> Result<Response, ContractError> {
     use ValidatorProposal::*;
     // anyone can trigger this if the vote passed
-    let proposal = mark_executed::<ValidatorProposal>(deps, env, proposal_id)?;
+    let proposal = mark_executed::<ValidatorProposal>(deps.storage, env, proposal_id)?;
 
-    let msg = match proposal.proposal {
-        RegisterUpgrade { name, height, info } => SubMsg::new(TgradeMsg::ExecuteGovProposal {
-            title: proposal.title,
-            description: proposal.description,
-            proposal: GovProposal::RegisterUpgrade { name, height, info },
-        }),
-        CancelUpgrade {} => SubMsg::new(TgradeMsg::ExecuteGovProposal {
-            title: proposal.title,
-            description: proposal.description,
-            proposal: GovProposal::CancelUpgrade {},
-        }),
-        PinCodes(code_ids) => SubMsg::new(TgradeMsg::ExecuteGovProposal {
-            title: proposal.title,
-            description: proposal.description,
-            proposal: GovProposal::PinCodes { code_ids },
-        }),
-        UnpinCodes(code_ids) => SubMsg::new(TgradeMsg::ExecuteGovProposal {
-            title: proposal.title,
-            description: proposal.description,
-            proposal: GovProposal::UnpinCodes { code_ids },
-        }),
+    let mut res = Response::new();
+
+    match proposal.proposal {
+        RegisterUpgrade { name, height, info } => {
+            res = res.add_message(TgradeMsg::ExecuteGovProposal {
+                title: proposal.title,
+                description: proposal.description,
+                proposal: GovProposal::RegisterUpgrade { name, height, info },
+            })
+        }
+        CancelUpgrade {} => {
+            res = res.add_message(TgradeMsg::ExecuteGovProposal {
+                title: proposal.title,
+                description: proposal.description,
+                proposal: GovProposal::CancelUpgrade {},
+            })
+        }
+        PinCodes(code_ids) => {
+            res = res.add_message(TgradeMsg::ExecuteGovProposal {
+                title: proposal.title,
+                description: proposal.description,
+                proposal: GovProposal::PinCodes { code_ids },
+            })
+        }
+        UnpinCodes(code_ids) => {
+            res = res.add_message(TgradeMsg::ExecuteGovProposal {
+                title: proposal.title,
+                description: proposal.description,
+                proposal: GovProposal::UnpinCodes { code_ids },
+            })
+        }
         UpdateConsensusBlockParams { max_bytes, max_gas } => {
-            SubMsg::new(TgradeMsg::ConsensusParams(ConsensusParams {
+            res = res.add_message(TgradeMsg::ConsensusParams(ConsensusParams {
                 block: Some(BlockParams { max_bytes, max_gas }),
                 evidence: None,
             }))
@@ -146,61 +167,67 @@ pub fn execute_execute(
             max_age_num_blocks,
             max_age_duration,
             max_bytes,
-        } => SubMsg::new(TgradeMsg::ConsensusParams(ConsensusParams {
-            block: None,
-            evidence: Some(EvidenceParams {
-                max_age_num_blocks,
-                max_age_duration,
-                max_bytes,
-            }),
-        })),
+        } => {
+            res = res.add_message(TgradeMsg::ConsensusParams(ConsensusParams {
+                block: None,
+                evidence: Some(EvidenceParams {
+                    max_age_num_blocks,
+                    max_age_duration,
+                    max_bytes,
+                }),
+            }))
+        }
         MigrateContract {
             contract,
             code_id,
             migrate_msg,
-        } => SubMsg::new(WasmMsg::Migrate {
-            contract_addr: contract,
-            new_code_id: code_id,
-            msg: migrate_msg,
-        }),
+        } => {
+            res = res.add_message(WasmMsg::Migrate {
+                contract_addr: contract,
+                new_code_id: code_id,
+                msg: migrate_msg,
+            })
+        }
+        Text {} => execute_text(deps, proposal_id, proposal)?,
     };
 
-    Ok(Response::new()
+    Ok(res
         .add_attribute("action", "execute")
-        .add_attribute("sender", info.sender)
-        .add_submessage(msg))
+        .add_attribute("sender", info.sender))
 }
 
 fn align_limit(limit: Option<u32>) -> usize {
     // settings for pagination
-    const MAX_LIMIT: u32 = 30;
-    const DEFAULT_LIMIT: u32 = 10;
+    const MAX_LIMIT: u32 = 100;
+    const DEFAULT_LIMIT: u32 = 30;
 
     limit.unwrap_or(DEFAULT_LIMIT).min(MAX_LIMIT) as _
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> StdResult<Binary> {
+pub fn query(deps: Deps<TgradeQuery>, env: Env, msg: QueryMsg) -> StdResult<Binary> {
     use QueryMsg::*;
 
     match msg {
         Rules {} => to_binary(&query_rules(deps)?),
-        Proposal { proposal_id } => to_binary(&query_proposal::<ValidatorProposal>(
+        Proposal { proposal_id } => to_binary(&query_proposal::<ValidatorProposal, TgradeQuery>(
             deps,
             env,
             proposal_id,
         )?),
         Vote { proposal_id, voter } => to_binary(&query_vote(deps, proposal_id, voter)?),
-        ListProposals { start_after, limit } => to_binary(&list_proposals::<ValidatorProposal>(
-            deps,
-            env,
-            start_after,
-            align_limit(limit),
-        )?),
+        ListProposals { start_after, limit } => {
+            to_binary(&list_proposals::<ValidatorProposal, TgradeQuery>(
+                deps,
+                env,
+                start_after,
+                align_limit(limit),
+            )?)
+        }
         ReverseProposals {
             start_before,
             limit,
-        } => to_binary(&reverse_proposals::<ValidatorProposal>(
+        } => to_binary(&reverse_proposals::<ValidatorProposal, TgradeQuery>(
             deps,
             env,
             start_before,
@@ -216,21 +243,38 @@ pub fn query(deps: Deps, env: Env, msg: QueryMsg) -> StdResult<Binary> {
             start_after,
             align_limit(limit),
         )?),
+        ListVotesByVoter {
+            voter,
+            start_after,
+            limit,
+        } => to_binary(&list_votes_by_voter(
+            deps,
+            voter,
+            start_after,
+            align_limit(limit),
+        )?),
         Voter { address } => to_binary(&query_voter(deps, address)?),
         ListVoters { start_after, limit } => to_binary(&list_voters(deps, start_after, limit)?),
         GroupContract {} => to_binary(&query_group_contract(deps)?),
+        ListTextProposals { start_after, limit } => {
+            to_binary(&list_text_proposals(deps, start_after, align_limit(limit))?)
+        }
     }
 }
 
 #[cfg_attr(not(feature = "library"), entry_point)]
-pub fn sudo(deps: DepsMut, _env: Env, msg: TgradeSudoMsg) -> Result<Response, ContractError> {
+pub fn sudo(
+    _deps: DepsMut<TgradeQuery>,
+    _env: Env,
+    msg: TgradeSudoMsg,
+) -> Result<Response, ContractError> {
     match msg {
-        TgradeSudoMsg::PrivilegeChange(change) => Ok(privilege_change(deps, change)),
+        TgradeSudoMsg::PrivilegeChange(change) => Ok(privilege_change(change)),
         _ => Err(ContractError::UnsupportedSudoType {}),
     }
 }
 
-fn privilege_change(_deps: DepsMut, change: PrivilegeChangeMsg) -> Response {
+fn privilege_change(change: PrivilegeChangeMsg) -> Response {
     match change {
         PrivilegeChangeMsg::Promoted {} => {
             let msgs = request_privileges(&[
@@ -252,21 +296,22 @@ pub fn migrate(deps: DepsMut, _env: Env, _msg: Empty) -> Result<Response, Contra
 #[cfg(test)]
 mod tests {
     use cosmwasm_std::{
-        testing::{mock_dependencies, mock_env, mock_info},
+        testing::{mock_env, mock_info},
         Addr, CosmosMsg, Decimal, SubMsg,
     };
     use tg_utils::Expiration;
     use tg_voting_contract::state::{proposals, Proposal, Votes, VotingRules};
 
     use super::*;
-    use cw3::Status;
+    use tg3::Status;
+    use tg_bindings_test::mock_deps_tgrade;
 
     #[derive(serde::Serialize)]
     struct DummyMigrateMsg {}
 
     #[test]
     fn register_migrate() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -275,6 +320,7 @@ mod tests {
                 &Proposal {
                     title: "MigrateContract".to_owned(),
                     description: "MigrateContract testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     // Aaaaall the seconds
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
@@ -290,7 +336,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -314,7 +360,7 @@ mod tests {
 
     #[test]
     fn register_cancel_upgrade() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -323,6 +369,7 @@ mod tests {
                 &Proposal {
                     title: "CancelUpgrade".to_owned(),
                     description: "CancelUpgrade testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
                     proposal: ValidatorProposal::CancelUpgrade {},
@@ -333,7 +380,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -359,7 +406,7 @@ mod tests {
 
     #[test]
     fn register_pin_codes() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -368,6 +415,7 @@ mod tests {
                 &Proposal {
                     title: "PinCodes".to_owned(),
                     description: "PinCodes testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
                     proposal: ValidatorProposal::PinCodes(vec![]),
@@ -378,7 +426,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -404,7 +452,7 @@ mod tests {
 
     #[test]
     fn register_unpin_codes() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -413,6 +461,7 @@ mod tests {
                 &Proposal {
                     title: "UnpinCodes".to_owned(),
                     description: "UnpinCodes testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
                     proposal: ValidatorProposal::UnpinCodes(vec![]),
@@ -423,7 +472,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -449,7 +498,7 @@ mod tests {
 
     #[test]
     fn update_consensus_block_params() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -458,6 +507,7 @@ mod tests {
                 &Proposal {
                     title: "UnpinCodes".to_owned(),
                     description: "UnpinCodes testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
                     proposal: ValidatorProposal::UpdateConsensusBlockParams {
@@ -471,7 +521,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -499,7 +549,7 @@ mod tests {
 
     #[test]
     fn update_consensus_evidence_params() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         proposals()
             .save(
@@ -508,6 +558,7 @@ mod tests {
                 &Proposal {
                     title: "UnpinCodes".to_owned(),
                     description: "UnpinCodes testing proposal".to_owned(),
+                    created_by: "mock_person".to_owned(),
                     start_height: env.block.height,
                     expires: Expiration::at_timestamp(env.block.time.plus_seconds(66666)),
                     proposal: ValidatorProposal::UpdateConsensusEvidenceParams {
@@ -522,7 +573,7 @@ mod tests {
                         threshold: Decimal::percent(40),
                         allow_end_early: true,
                     },
-                    total_weight: 20,
+                    total_points: 20,
                     votes: Votes {
                         yes: 20,
                         no: 0,
@@ -551,7 +602,7 @@ mod tests {
 
     #[test]
     fn query_group_contract() {
-        let mut deps = mock_dependencies();
+        let mut deps = mock_deps_tgrade();
         let env = mock_env();
         let rules = VotingRules {
             voting_period: 1,
